@@ -11,14 +11,28 @@ H2 state machine it processes the data and returns a list of Event objects.
 """
 import binascii
 
-from .settings import ChangedSetting
+from .settings import ChangedSetting, _setting_code_from_int
 
 
-class RequestReceived(object):
+class Event(object):
+    """
+    Base class for h2 events.
+    """
+    pass
+
+
+class RequestReceived(Event):
     """
     The RequestReceived event is fired whenever request headers are received.
     This event carries the HTTP headers for the given request and the stream ID
     of the new stream.
+
+    .. versionchanged:: 2.3.0
+       Changed the type of ``headers`` to :class:`HeaderTuple
+       <hpack:hpack.HeaderTuple>`. This has no effect on current users.
+
+    .. versionchanged:: 2.4.0
+       Added ``stream_ended`` and ``priority_updated`` properties.
     """
     def __init__(self):
         #: The Stream ID for the stream this request was made on.
@@ -27,17 +41,38 @@ class RequestReceived(object):
         #: The request headers.
         self.headers = None
 
+        #: If this request also ended the stream, the associated
+        #: :class:`StreamEnded <h2.events.StreamEnded>` event will be available
+        #: here.
+        #:
+        #: .. versionadded:: 2.4.0
+        self.stream_ended = None
+
+        #: If this request also had associated priority information, the
+        #: associated :class:`PriorityUpdated <h2.events.PriorityUpdated>`
+        #: event will be available here.
+        #:
+        #: .. versionadded:: 2.4.0
+        self.priority_updated = None
+
     def __repr__(self):
         return "<RequestReceived stream_id:%s, headers:%s>" % (
             self.stream_id, self.headers
         )
 
 
-class ResponseReceived(object):
+class ResponseReceived(Event):
     """
-    The ResponseReceived event is fired whenever request headers are received.
+    The ResponseReceived event is fired whenever response headers are received.
     This event carries the HTTP headers for the given response and the stream
     ID of the new stream.
+
+    .. versionchanged:: 2.3.0
+       Changed the type of ``headers`` to :class:`HeaderTuple
+       <hpack:hpack.HeaderTuple>`. This has no effect on current users.
+
+   .. versionchanged:: 2.4.0
+      Added ``stream_ended`` and ``priority_updated`` properties.
     """
     def __init__(self):
         #: The Stream ID for the stream this response was made on.
@@ -46,13 +81,27 @@ class ResponseReceived(object):
         #: The response headers.
         self.headers = None
 
+        #: If this response also ended the stream, the associated
+        #: :class:`StreamEnded <h2.events.StreamEnded>` event will be available
+        #: here.
+        #:
+        #: .. versionadded:: 2.4.0
+        self.stream_ended = None
+
+        #: If this response also had associated priority information, the
+        #: associated :class:`PriorityUpdated <h2.events.PriorityUpdated>`
+        #: event will be available here.
+        #:
+        #: .. versionadded:: 2.4.0
+        self.priority_updated = None
+
     def __repr__(self):
         return "<ResponseReceived stream_id:%s, headers:%s>" % (
             self.stream_id, self.headers
         )
 
 
-class TrailersReceived(object):
+class TrailersReceived(Event):
     """
     The TrailersReceived event is fired whenever trailers are received on a
     stream. Trailers are a set of headers sent after the body of the
@@ -60,6 +109,13 @@ class TrailersReceived(object):
     ahead of time (e.g. content-length). This event carries the HTTP header
     fields that form the trailers and the stream ID of the stream on which they
     were received.
+
+    .. versionchanged:: 2.3.0
+       Changed the type of ``headers`` to :class:`HeaderTuple
+       <hpack:hpack.HeaderTuple>`. This has no effect on current users.
+
+    .. versionchanged:: 2.4.0
+       Added ``stream_ended`` and ``priority_updated`` properties.
     """
     def __init__(self):
         #: The Stream ID for the stream on which these trailers were received.
@@ -68,13 +124,82 @@ class TrailersReceived(object):
         #: The trailers themselves.
         self.headers = None
 
+        #: Trailers always end streams. This property has the associated
+        #: :class:`StreamEnded <h2.events.StreamEnded>` in it.
+        #:
+        #: .. versionadded:: 2.4.0
+        self.stream_ended = None
+
+        #: If the trailers also set associated priority information, the
+        #: associated :class:`PriorityUpdated <h2.events.PriorityUpdated>`
+        #: event will be available here.
+        #:
+        #: .. versionadded:: 2.4.0
+        self.priority_updated = None
+
     def __repr__(self):
         return "<TrailersReceived stream_id:%s, headers:%s>" % (
             self.stream_id, self.headers
         )
 
 
-class InformationalResponseReceived(object):
+class _HeadersSent(Event):
+    """
+    The _HeadersSent event is fired whenever headers are sent.
+
+    This is an internal event, used to determine validation steps on
+    outgoing header blocks.
+    """
+    pass
+
+
+class _ResponseSent(_HeadersSent):
+    """
+    The _ResponseSent event is fired whenever response headers are sent
+    on a stream.
+
+    This is an internal event, used to determine validation steps on
+    outgoing header blocks.
+    """
+    pass
+
+
+class _RequestSent(_HeadersSent):
+    """
+    The _RequestSent event is fired whenever request headers are sent
+    on a stream.
+
+    This is an internal event, used to determine validation steps on
+    outgoing header blocks.
+    """
+    pass
+
+
+class _TrailersSent(_HeadersSent):
+    """
+    The _TrailersSent event is fired whenever trailers are sent on a
+    stream. Trailers are a set of headers sent after the body of the
+    request/response, and are used to provide information that wasn't known
+    ahead of time (e.g. content-length).
+
+    This is an internal event, used to determine validation steps on
+    outgoing header blocks.
+    """
+    pass
+
+
+class _PushedRequestSent(_HeadersSent):
+    """
+    The _PushedRequestSent event is fired whenever pushed request headers are
+    sent.
+
+    This is an internal event, used to determine validation steps on outgoing
+    header blocks.
+    """
+    pass
+
+
+class InformationalResponseReceived(Event):
     """
     The InformationalResponseReceived event is fired when an informational
     response (that is, one whose status code is a 1XX code) is received from
@@ -88,6 +213,13 @@ class InformationalResponseReceived(object):
     1XX status code.
 
     .. versionadded:: 2.2.0
+
+    .. versionchanged:: 2.3.0
+       Changed the type of ``headers`` to :class:`HeaderTuple
+       <hpack:hpack.HeaderTuple>`. This has no effect on current users.
+
+    .. versionchanged:: 2.4.0
+       Added ``priority_updated`` property.
     """
     def __init__(self):
         #: The Stream ID for the stream this informational response was made
@@ -97,17 +229,27 @@ class InformationalResponseReceived(object):
         #: The headers for this informational response.
         self.headers = None
 
+        #: If this response also had associated priority information, the
+        #: associated :class:`PriorityUpdated <h2.events.PriorityUpdated>`
+        #: event will be available here.
+        #:
+        #: .. versionadded:: 2.4.0
+        self.priority_updated = None
+
     def __repr__(self):
         return "<InformationalResponseReceived stream_id:%s, headers:%s>" % (
             self.stream_id, self.headers
         )
 
 
-class DataReceived(object):
+class DataReceived(Event):
     """
     The DataReceived event is fired whenever data is received on a stream from
     the remote peer. The event carries the data itself, and the stream ID on
     which the data was received.
+
+    .. versionchanged:: 2.4.0
+       Added ``stream_ended`` property.
     """
     def __init__(self):
         #: The Stream ID for the stream this data was received on.
@@ -122,6 +264,13 @@ class DataReceived(object):
         #: than ``len(data)``.
         self.flow_controlled_length = None
 
+        #: If this data chunk also completed the stream, the associated
+        #: :class:`StreamEnded <h2.events.StreamEnded>` event will be available
+        #: here.
+        #:
+        #: .. versionadded:: 2.4.0
+        self.stream_ended = None
+
     def __repr__(self):
         return (
             "<DataReceived stream_id:%s, "
@@ -134,7 +283,7 @@ class DataReceived(object):
         )
 
 
-class WindowUpdated(object):
+class WindowUpdated(Event):
     """
     The WindowUpdated event is fired whenever a flow control window changes
     size. HTTP/2 defines flow control windows for connections and streams: this
@@ -156,7 +305,7 @@ class WindowUpdated(object):
         )
 
 
-class RemoteSettingsChanged(object):
+class RemoteSettingsChanged(Event):
     """
     The RemoteSettingsChanged event is fired whenever the remote peer changes
     its settings. It contains a complete inventory of changed settings,
@@ -169,7 +318,7 @@ class RemoteSettingsChanged(object):
     When this event is received, the caller should confirm that the new
     settings are acceptable. If they are not acceptable, the user should close
     the connection with the error code :data:`PROTOCOL_ERROR
-    <h2.errors.PROTOCOL_ERROR>`.
+    <h2.errors.ErrorCodes.PROTOCOL_ERROR>`.
 
     .. versionchanged:: 2.0.0
        Prior to this version the user needed to acknowledge settings changes.
@@ -194,6 +343,7 @@ class RemoteSettingsChanged(object):
         """
         e = cls()
         for setting, new_value in new_settings.items():
+            setting = _setting_code_from_int(setting)
             original_value = old_settings.get(setting)
             change = ChangedSetting(setting, original_value, new_value)
             e.changed_settings[setting] = change
@@ -201,12 +351,12 @@ class RemoteSettingsChanged(object):
         return e
 
     def __repr__(self):
-        return "<RemoteSettingsChanged changed_settings:%s>" % (
-            self.changed_settings,
+        return "<RemoteSettingsChanged changed_settings:{%s}>" % (
+            ", ".join(repr(cs) for cs in self.changed_settings.values()),
         )
 
 
-class PingAcknowledged(object):
+class PingAcknowledged(Event):
     """
     The PingAcknowledged event is fired whenever a user-emitted PING is
     acknowledged. This contains the data in the ACK'ed PING, allowing the
@@ -222,7 +372,7 @@ class PingAcknowledged(object):
         )
 
 
-class StreamEnded(object):
+class StreamEnded(Event):
     """
     The StreamEnded event is fired whenever a stream is ended by a remote
     party. The stream may not be fully closed if it has not been closed
@@ -236,7 +386,7 @@ class StreamEnded(object):
         return "<StreamEnded stream_id:%s>" % self.stream_id
 
 
-class StreamReset(object):
+class StreamReset(Event):
     """
     The StreamReset event is fired in two situations. The first is when the
     remote party forcefully resets the stream. The second is when the remote
@@ -250,7 +400,8 @@ class StreamReset(object):
         #: The Stream ID of the stream that was reset.
         self.stream_id = None
 
-        #: The error code given.
+        #: The error code given. Either one of :class:`ErrorCodes
+        #: <h2.errors.ErrorCodes>` or ``int``
         self.error_code = None
 
         #: Whether the remote peer sent a RST_STREAM or we did.
@@ -262,7 +413,7 @@ class StreamReset(object):
         )
 
 
-class PushedStreamReceived(object):
+class PushedStreamReceived(Event):
     """
     The PushedStreamReceived event is fired whenever a pushed stream has been
     received from a remote peer. The event carries on it the new stream ID, the
@@ -289,7 +440,7 @@ class PushedStreamReceived(object):
         )
 
 
-class SettingsAcknowledged(object):
+class SettingsAcknowledged(Event):
     """
     The SettingsAcknowledged event is fired whenever a settings ACK is received
     from the remote peer. The event carries on it the settings that were
@@ -303,12 +454,12 @@ class SettingsAcknowledged(object):
         self.changed_settings = {}
 
     def __repr__(self):
-        return "<SettingsAcknowledged changed_settings:%s>" % (
-            self.changed_settings,
+        return "<SettingsAcknowledged changed_settings:{%s}>" % (
+            ", ".join(repr(cs) for cs in self.changed_settings.values()),
         )
 
 
-class PriorityUpdated(object):
+class PriorityUpdated(Event):
     """
     The PriorityUpdated event is fired whenever a stream sends updated priority
     information. This can occur when the stream is opened, or at any time
@@ -346,7 +497,7 @@ class PriorityUpdated(object):
         )
 
 
-class ConnectionTerminated(object):
+class ConnectionTerminated(Event):
     """
     The ConnectionTerminated event is fired when a connection is torn down by
     the remote peer using a GOAWAY frame. Once received, no further action may
@@ -354,7 +505,7 @@ class ConnectionTerminated(object):
     """
     def __init__(self):
         #: The error code cited when tearing down the connection. Should be
-        #: one of :data:`H2ERRORS <h2.errors.H2_ERRORS>`, but may not be if
+        #: one of :class:`ErrorCodes <h2.errors.ErrorCodes>`, but may not be if
         #: unknown HTTP/2 extensions are being used.
         self.error_code = None
 
@@ -379,6 +530,72 @@ class ConnectionTerminated(object):
         )
 
 
+class AlternativeServiceAvailable(Event):
+    """
+    The AlternativeServiceAvailable event is fired when the remote peer
+    advertises an `RFC 7838 <https://tools.ietf.org/html/rfc7838>`_ Alternative
+    Service using an ALTSVC frame.
+
+    This event always carries the origin to which the ALTSVC information
+    applies. That origin is either supplied by the server directly, or inferred
+    by hyper-h2 from the ``:authority`` pseudo-header field that was sent by
+    the user when initiating a given stream.
+
+    This event also carries what RFC 7838 calls the "Alternative Service Field
+    Value", which is formatted like a HTTP header field and contains the
+    relevant alternative service information. Hyper-h2 does not parse or in any
+    way modify that information: the user is required to do that.
+
+    This event can only be fired on the client end of a connection.
+
+    .. versionadded:: 2.3.0
+    """
+    def __init__(self):
+        #: The origin to which the alternative service field value applies.
+        #: This field is either supplied by the server directly, or inferred by
+        #: hyper-h2 from the ``:authority`` pseudo-header field that was sent
+        #: by the user when initiating the stream on which the frame was
+        #: received.
+        self.origin = None
+
+        #: The ALTSVC field value. This contains information about the HTTP
+        #: alternative service being advertised by the server. Hyper-h2 does
+        #: not parse this field: it is left exactly as sent by the server. The
+        #: structure of the data in this field is given by `RFC 7838 Section 3
+        #: <https://tools.ietf.org/html/rfc7838#section-3>`_.
+        self.field_value = None
+
+    def __repr__(self):
+        return (
+            "<AlternativeServiceAvailable origin:%s, field_value:%s>" % (
+                self.origin.decode('utf-8', 'ignore'),
+                self.field_value.decode('utf-8', 'ignore'),
+            )
+        )
+
+
+class UnknownFrameReceived(Event):
+    """
+    The UnknownFrameReceived event is fired when the remote peer sends a frame
+    that hyper-h2 does not understand. This occurs primarily when the remote
+    peer is employing HTTP/2 extensions that hyper-h2 doesn't know anything
+    about.
+
+    RFC 7540 requires that HTTP/2 implementations ignore these frames. hyper-h2
+    does so. However, this event is fired to allow implementations to perform
+    special processing on those frames if needed (e.g. if the implementation
+    is capable of handling the frame itself).
+
+    .. versionadded:: 2.7.0
+    """
+    def __init__(self):
+        #: The hyperframe Frame object that encapsulates the received frame.
+        self.frame = None
+
+    def __repr__(self):
+        return "<UnknownFrameReceived>"
+
+
 def _bytes_representation(data):
     """
     Converts a bytestring into something that is safe to print on all Python
